@@ -3,40 +3,28 @@ package Net::LDAP::SimpleServer::ProtocolHandler;
 use strict;
 use warnings;
 
-use common::sense;
-
 # ABSTRACT: LDAP protocol handler used with Net::LDAP::SimpleServer
 
-our $VERSION = '0.0.15';    # VERSION
+our $VERSION = '0.0.16';    # VERSION
 
 use Net::LDAP::Server;
 use base 'Net::LDAP::Server';
-use fields qw(store root_dn root_pw);
+use fields qw(store root_dn root_pw allow_anon);
 
 use Carp;
 use Net::LDAP::LDIF;
 use Net::LDAP::Util qw{canonical_dn};
 use Net::LDAP::FilterMatch;
 
-use Scalar::Util qw{blessed reftype looks_like_number};
+use Net::LDAP::Constant (
+    qw/LDAP_SUCCESS LDAP_AUTH_UNKNOWN LDAP_INVALID_CREDENTIALS/,
+    qw/LDAP_AUTH_METHOD_NOT_SUPPORTED/ );
+
+use Scalar::Util qw{reftype};
 use UNIVERSAL::isa;
 
 use Data::Dumper;
 
-my %_ldap_cache = ();
-
-sub _get_ldap_constant {
-    my $code = shift;
-    return $code if looks_like_number($code);
-    return $_ldap_cache{$code} if exists $_ldap_cache{$code};
-    ## no critic
-    return $_ldap_cache{$code} = eval qq{
-        use Net::LDAP::Constant qw|$code|;
-        $code;
-    };
-    ## use critic
-}
-
 sub _make_result {
     my $code = shift;
     my $dn   = shift || '';
@@ -45,31 +33,29 @@ sub _make_result {
     return {
         matchedDN    => $dn,
         errorMessage => $msg,
-        resultCode   => _get_ldap_constant($code),
+        resultCode   => $code,
     };
 }
 
 sub new {
     my $class  = shift;
-    my $params = shift;
-    my $self   = $class->SUPER::new(@_);
+    my $params = shift || croak 'Must pass parameters!';
+    my $self   = $class->SUPER::new( $params->{input}, $params->{output} );
 
-    croak 'First parameter must be an ARRAYREF'
-      unless reftype($params) eq 'HASH';
-
-    croak 'Must pass store!' unless exists $params->{store};
-    croak 'Not an object!'   unless blessed( $params->{store} );
-    croak 'Not a LDIFStore!'
+    croak 'Parameter must be a HASHREF' unless reftype($params) eq 'HASH';
+    croak 'Must pass option {store}' unless exists $params->{store};
+    croak 'Not a LDIFStore'
       unless $params->{store}->isa('Net::LDAP::SimpleServer::LDIFStore');
 
-    croak 'Must pass root_dn!' unless exists $params->{root_dn};
-
+    croak 'Must pass option {root_dn}' unless exists $params->{root_dn};
+    croak 'Option {root_dn} can not be empty' unless $params->{root_dn};
     croak 'Invalid root DN'
       unless my $canon_dn = canonical_dn( $params->{root_dn} );
 
-    $self->{store}   = $params->{store};
-    $self->{root_dn} = $canon_dn;
-    $self->{root_pw} = $params->{root_pw} || '';
+    $self->{store}      = $params->{store};
+    $self->{root_dn}    = $canon_dn;
+    $self->{root_pw}    = $params->{root_pw};
+    $self->{allow_anon} = $params->{allow_anon};
     chomp( $self->{root_pw} );
 
     return $self;
@@ -82,36 +68,36 @@ sub unbind {
     $self->{root_dn} = undef;
     $self->{root_pw} = undef;
 
-    return _make_result(qw/LDAP_SUCCESS/);
+    return _make_result(LDAP_SUCCESS);
 }
 
 sub bind {    ## no critic
-
-    #    my $r = _bind(@_);
-    #    print STDERR q{response = } . Dumper($r);
-    #    return $r;
-    #}
-    #
-    #sub _bind {
     my ( $self, $request ) = @_;
 
     #print STDERR '=' x 70 . "\n";
+    #print STDERR Dumper($self);
     #print STDERR Dumper($request);
-    my $ok = _make_result(qw/LDAP_SUCCESS/);
-    return $ok unless $request->{name};
+    my $ok = _make_result(LDAP_SUCCESS);
+
+    if (    not $request->{name}
+        and exists $request->{authentication}->{simple}
+        and $self->{allow_anon} )
+    {
+        return $ok;
+    }
 
     #print STDERR qq{not anonymous\n};
     # As of now, accepts only simple authentication
-    return _make_result(qw/LDAP_AUTH_UNKNOWN/)
+    return _make_result(LDAP_AUTH_METHOD_NOT_SUPPORTED)
       unless exists $request->{authentication}->{simple};
 
     #print STDERR qq{is simple authentication\n};
-    return _make_result(qw/LDAP_INVALID_CREDENTIALS/)
+    return _make_result(LDAP_INVALID_CREDENTIALS)
       unless my $binddn = canonical_dn( $request->{name} );
 
     #print STDERR qq#binddn is ok ($request->{name}) => ($binddn)\n#;
     #print STDERR qq#handler dn is $self->{root_dn}\n#;
-    return _make_result(qw/LDAP_INVALID_CREDENTIALS/)
+    return _make_result(LDAP_INVALID_CREDENTIALS)
       unless uc($binddn) eq uc( $self->{root_dn} );
 
     #print STDERR qq{binddn is good\n};
@@ -119,7 +105,7 @@ sub bind {    ## no critic
     chomp($bindpw);
 
     #print STDERR qq|comparing ($bindpw) eq ($self->{root_pw})\n|;
-    return _make_result(qw/LDAP_INVALID_CREDENTIALS/)
+    return _make_result(LDAP_INVALID_CREDENTIALS)
       unless $bindpw eq $self->{root_pw};
 
     return $ok;
@@ -135,8 +121,9 @@ sub _match {
 sub search {
     my ( $self, $request ) = @_;
 
-    my $list   = $self->{store}->list;
-    my $basedn = $request->{baseObject};
+    my $list = $self->{store}->list;
+
+    #my $basedn = $request->{baseObject};
 
     #print STDERR '=' x 50 . "\n";
     #print STDERR Dumper($request);
@@ -146,12 +133,13 @@ sub search {
 
     #print STDERR Dumper($res);
 
-    return ( _make_result(qw/LDAP_SUCCESS/), @{$res} );
+    return ( _make_result(LDAP_SUCCESS), @{$res} );
 }
 
 1;    # Magic true value required at end of module
 
 
+
 =pod
 
 =encoding utf-8
@@ -162,7 +150,7 @@ Net::LDAP::SimpleServer::ProtocolHandler - LDAP protocol handler used with Net::
 
 =head1 VERSION
 
-version 0.0.15
+version 0.0.16
 
 =head1 SYNOPSIS
 
@@ -220,7 +208,7 @@ Alexei Znamensky <russoz@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Alexei Znamensky.
+This software is copyright (c) 2012 by Alexei Znamensky.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -258,3 +246,4 @@ DAMAGES.
 
 __END__
 
+
